/* pingpong
   Exchanges messages between two MPI processes
   Run with mpirun -np 2 */

#include <stdio.h>
#include <string.h>
#include <mpi.h>

#define MSG_MAX_LEN 10
#define NB_MESS 20000000

int main(int argc, char *argv[]){

  int myRank, otherRank, len;
  char msg[MSG_MAX_LEN];
  MPI_Status status;
  int tag = 1;
  // MPI_Wtime() *should* return a long but it returns EPOCH...
  unsigned long int startTime, endTime;

  /* Initialize MPI and get the rank */
  MPI_Init(&argc, &argv);

  MPI_Comm_rank(MPI_COMM_WORLD, &myRank);
  otherRank = 1-myRank;

  startTime = MPI_Wtime();

  /* Set the message string
     Send the first message if in process 0 */
  if (myRank == 0) {
    strcpy(msg, "ping");
    len = strlen(msg)+1;
    MPI_Send(msg, len, MPI_CHAR, otherRank, tag++, MPI_COMM_WORLD);
  }
  else {
    strcpy(msg, "pong");
    len = strlen(msg)+1;
  }

  /* Answer messages while NB_MESS messages haven't been sent */
  for(;;){
    MPI_Recv(msg, MSG_MAX_LEN, MPI_CHAR, otherRank, tag++, MPI_COMM_WORLD, &status);
    //-- printf("%d> %s (%d)\n", myRank, msg, tag-1);
    MPI_Send(msg, len, MPI_CHAR, otherRank, tag++, MPI_COMM_WORLD);
    if (tag > NB_MESS) break;
  }

  /* Print time elapsed, clean and exit */
  endTime = MPI_Wtime();
  printf("%d> I took %ds to complete.\n", myRank, endTime-startTime);

  MPI_Finalize();
  return 0;

}
